@@ -1,5 +1,5 @@
 /*++ @file
-  Stub SEC that is called from the OS appliation that is the root of the emulator.
+  Stub SEC that is called from the OS application that is the root of the emulator.
 
   The OS application will call the SEC with the PEI Entry Point API.
 
@@ -11,7 +11,6 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 #ifndef __SEC_H___
 #define __SEC_H___
 
-
 #include <PiPei.h>
 #include <Library/EmuMagicPageLib.h>
 #include <Library/DebugLib.h>
@@ -21,25 +20,13 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 
 #include <Ppi/TemporaryRamSupport.h>
 
-
-//
-// I think this shold be defined in a MdePkg include file?
-//
-VOID
-EFIAPI
-ProcessLibraryConstructorList (
-  VOID
-  );
-
 EFI_STATUS
 EFIAPI
 SecTemporaryRamSupport (
-  IN CONST EFI_PEI_SERVICES   **PeiServices,
-  IN EFI_PHYSICAL_ADDRESS     TemporaryMemoryBase,
-  IN EFI_PHYSICAL_ADDRESS     PermanentMemoryBase,
-  IN UINTN                    CopySize
+  IN CONST EFI_PEI_SERVICES  **PeiServices,
+  IN EFI_PHYSICAL_ADDRESS    TemporaryMemoryBase,
+  IN EFI_PHYSICAL_ADDRESS    PermanentMemoryBase,
+  IN UINTN                   CopySize
   );
 
-
 #endif
-
