@@ -6,6 +6,8 @@
 
 #include <clk.h>
 #include <common.h>
+#include <hang.h>
+#include <init.h>
 #include <spl.h>
 
 #include "init.h"
