@@ -1,7 +1,7 @@
 /*
  * MIPS cpu parameters for qemu.
  *
- * SPDX-License-Identifier: LGPL-2.0+
+ * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #ifndef MIPS_CPU_PARAM_H
@@ -24,6 +24,7 @@
 #endif
 #endif
 #define TARGET_PAGE_BITS 12
-#define NB_MMU_MODES 4
+
+#define TCG_GUEST_DEFAULT_MO (0)
 
 #endif
